@@ -1,20 +1,18 @@
 #
 # This file is part of Config-Model-Systemd
 #
-# This software is Copyright (c) 2015-2017 by Dominique Dumont.
+# This software is Copyright (c) 2015-2020 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 # systemd tests for user
+use strict;
+use warnings;
 
-$home_for_test='/home/joe';
-$conf_dir = '~/.config/systemd/user/';
-$config_file_name = 'systemd-user';
-
-# list of tests. This modules looks for @tests global variable
-@tests = (
+# list of tests.
+my @tests = (
     {
         name => 'basic-service',
         backend_arg => 'gmail',
@@ -83,4 +81,10 @@ $config_file_name = 'systemd-user';
     }
 );
 
-1; # to keep Perl happy
+return {
+    tests => \@tests,
+    home_for_test=>'/home/joe',
+    conf_dir => '~/.config/systemd/user/',
+    config_file_name => 'systemd-user',
+}
+
